{ FLDISK.1A flash access unit

  Pascal Dornier 4/19/97 }

unit flash;

{$r-,i-,s-}

interface

procedure hex(i:word);		{ display hex byte }
procedure rom_page(pg:word); 	{ set page register, pg is address div 256 }
procedure rom_wait;		{ wait until write operation completed }
procedure rom_find;		{ find ROM segment }
procedure rom_exit;		{ close ROM }
procedure rom_id;		{ display manufacturer ID }
procedure rom_unlock;		{ unlock write access }
procedure rom_zap;		{ chip erase }
procedure rom_write(var buf; page,len:word);	{ write page }
procedure rom_verif(var buf; page,len:word);	{ verify page }
procedure rom_read(var buf; page,len:word); 	{ read page }
procedure rom_erase(pg:word);	{ erase a ROM page }
procedure rom_erase_p(pg:word);	{ erase a ROM page, physical address }

implementation

const
  hx:array[0..15] of char = '0123456789ABCDEF';

type
  romtp=array[0..8191] of byte;

var
  rom:^romtp;
  page:^byte;	{ page register }
  pageofs:word;
  romseg:word;

(***** display hex byte *****)

procedure hex(i:word);
begin
  write(hx[i shr 4],hx[i and 15]);
end;

(***** set flash EPROM page register, calculate pageofs *****)

procedure rom_page(pg:word); { pg is address div 256 }
begin
  page^:=pg shr 5;
  pageofs:=swap(pg and $1f);
end;

(***** wait until write operation completed *****)

procedure rom_wait;
var
  i,j:byte;
  k:word;
begin
  i:=rom^[pageofs];
  k:=0;
  repeat
    dec(k);		{ time-out counter }
    if k=0 then exit;
    j:=i;
    i:=rom^[pageofs];
  until i=j;
end;

(***** clear page register, write protect *****)

procedure rom_exit;
begin
  rom^[0]:=$ff;		{ reset }
end;

(***** find ROM segment *****)

procedure rom_find;
var
  i,j:byte;
begin
  romseg:=$c800;
  repeat
    rom:=ptr(romseg+$0200,0);
    rom^[0]:=$90;		{ read ID }
    i:=rom^[0]; j:=rom^[1];
    if ((i=$bf) and (j=$04))	{ look for SST ID }
    then begin	
      rom^[0]:=$ff;		{ reset }
      write('Flash found at '); hex(hi(romseg)); hex(lo(romseg)); writeln;
      page:=ptr(romseg,0);
      exit;
    end;
    inc(romseg,$0800);
  until romseg=$e000;
  rom_exit;
  writeln('Flash EPROM not found !');
  halt(1);
end;

(***** display device ID *****)

procedure rom_id;
begin
  rom^[0]:=$90;		{ read ID }
  write('Manufacturer ID: '); hex(rom^[0]); writeln;
  write('Device ID:       '); hex(rom^[1]); writeln;
  rom^[0]:=$ff;		{ reset }
end;

(***** unlock write access *****)

procedure rom_unlock;
var
  i:byte;
begin
  page^:=0;		{ unlock write access }
  i:=rom^[$1823];
  i:=rom^[$1820];
  i:=rom^[$1822];
  i:=rom^[$0418];
  i:=rom^[$041B];
  i:=rom^[$0419];
  i:=rom^[$041A];
end;

(***** chip erase *****)

procedure rom_zap;
var
  i:word;
  error:boolean;
begin
  rom_page(0);		
  rom^[pageofs]:=$30;	{ chip erase setup command }
  rom^[pageofs]:=$30;	{ chip erase execute command }
  rom_wait;		{ wait until write complete }

  error:=false;
  for i:=0 to 2047 do begin	{ verify chip is erased }
    rom_page(i);
    asm
     	mov	ax,romseg
     	mov	es,ax
     	mov	di,pageofs
     	mov	cx,128
     	mov	ax,$ffff
     	repz	scasw
     	jz	@l1
     	mov	error,1
@l1:
    end;
  end;
  if error then begin
    writeln('Erase ERROR !'#7);
    halt(1);
  end;
end;

(***** write flash EPROM *****)

procedure rom_write(var buf; page,len:word);
var
  error:boolean;
begin
  rom_page(page);
  error:=false;
  asm
   	mov	ax,word ptr rom+2	{ destination segment }
   	mov	es,ax
   	mov	di,pageofs		{ destination offset }
   	push	ds
   	lds	si,buf			{ ^source data }
   	mov	cx,len
  @l1:	mov	al,[si]
        mov	byte ptr [es:di],$10	{ byte program command }
        mov	[es:di],al		{ write data }
        mov	dx,$ffff		{ time-out counter }
  @l2:	dec	dx
  	jz	@l3			{ :error }
  	cmp	[es:di],al		{ data written ? }
  	jnz	@l2
  	cmp	[es:di],al
  	jnz	@l2
	inc	si
	inc	di
	loop	@l1
	jmp	@l4
	
  @l3:	mov	error,1
  @l4:  pop	ds
  end;
  if error then begin
    write('Write ERROR !'#7);
    halt(1);
  end;
end;

(***** read flash EPROM *****)

procedure rom_read(var buf; page,len:word);
begin
  rom_page(page);
  move(rom^[pageofs],buf,len);
end;

(***** verify flash EPROM *****)

procedure rom_verif(var buf; page,len:word);
var
  i:word;
  b:array[0..32767] of byte absolute buf;
begin
  rom_page(page);
  for i:=0 to len-1 do
    if rom^[pageofs+i]<>b[i] then begin
      writeln('Verify ERROR !'#7);
      halt(1);
    end;
end;

(***** erase a ROM page *****)

procedure rom_erase(pg:word);
var
  error:boolean;
begin
  rom_page(pg);
  rom^[pageofs]:=$20;	{ sector erase setup command }
  rom^[pageofs]:=$d0;	{ sector erase execute command }
  rom_wait;		{ wait until write complete }
  error:=false;
  asm
	cld
     	mov	ax,romseg
     	mov	es,ax
     	mov	di,pageofs
     	mov	cx,128
     	mov	ax,$ffff
     	repz	scasw
     	jz	@l1
     	mov	error,1
@l1:
  end;
  if error then begin
    writeln('Erase ERROR !'#7);
    halt(1);
  end;
end;

(***** erase a ROM page, physical address *****)

procedure rom_erase_p(pg:word);
var
  error:boolean;
begin
  page^:=pg shr 5;
  pageofs:=swap(pg and $1f);
  rom^[pageofs]:=$20;	{ sector erase setup command }
  rom^[pageofs]:=$d0;	{ sector erase execute command }
  rom_wait;		{ wait until write complete }
  error:=false;
  asm
	cld
     	mov	ax,romseg
     	mov	es,ax
     	mov	di,pageofs
     	mov	cx,128
     	mov	ax,$ffff
     	repz	scasw
     	jz	@l1
     	mov	error,1
@l1:
  end;
  if error then begin
    writeln('Erase ERROR !'#7);
    halt(1);
  end;
end;

end.
